#define method1

struct VS_OUT
{
	float4 hPosition	: POSITION;
	float4 cPosition	: TEXCOORD1;
	float2 texCoord		: TEXCOORD0;
	float r				: TEXCOORD2;	
	float4 center		: TEXCOORD3;
	float4 Color		: TEXCOORD4;	
	float2 screenCoord	: TEXCOORD5;
};

VS_OUT HPS_Smoke_L_Depth_VS (float4 position : POSITION,	
		float4 texCoord : TEXCOORD0,
		float4 Color: COLOR0,
		uniform float baseRadius,
		uniform float4x4 worldView,
		uniform float4x4 Proj,
		uniform float4x4 worldViewProj)
{
	VS_OUT OUT;
	//texCoord.z = normalize(texCoord.z);
	//texCoord.w = normalize(texCoord.w);
    float2 offset = texCoord.zw * baseRadius;
	OUT.r = abs(offset.x);
	float4 cPosition;   
	cPosition = mul(worldView, position);
	OUT.center = cPosition;	
	cPosition.xy += offset;	
	OUT.cPosition = cPosition;
	OUT.cPosition.z *= -1;
	//OUT.cPosition.z = OUT.cPosition.z - OUT.r;	
	
    OUT.hPosition = mul( Proj, cPosition );
    float4 sPosition = OUT.hPosition / OUT.hPosition.w;
    OUT.screenCoord = (sPosition.xy + float2(1, 1)) / 2.0;
    OUT.screenCoord.y = 1.0 - OUT.screenCoord.y;
     
    OUT.texCoord = texCoord.xy;  
    OUT.Color = Color;  
	return OUT;
} 


float4 HPS_Smoke_L_Depth_PS(VS_OUT IN,
						uniform float4x4 Proj,
						uniform sampler2D Texture  : register(s0),
						uniform sampler2D depthTexture : register(s1),
						uniform float farplane,
						uniform float width = 800,
						uniform float height = 600
							) : COLOR
{
	float4 Color = 0;
	float4 impostor = tex2D(Texture, IN.texCoord);
	float f = 1.0 - impostor.g;
	float b = impostor.r;
	float alpha = 0;
	
	if(b == 0)
		discard;
	
	float2 halfpixel = float2(1.0 / width, 1.0 / height);
    IN.screenCoord += halfpixel;
   
	float sceneDepth = tex2D(depthTexture, IN.screenCoord).r;
	if(sceneDepth == 0)
		//discard;
		sceneDepth = farplane;
	
	float size = 2.0 * IN.r;
	float frontDepth = IN.cPosition.z + size * (f - 0.5);
	float backDepth = IN.cPosition.z +  size * (b - 0.5);
	
	alpha = (sceneDepth - frontDepth) / (backDepth - frontDepth);
	alpha = saturate(alpha) * impostor.a;
	
	//Color = float4(Color.g, 0, 0, 1);
	Color = float4(1, 1, 1, alpha) * IN.Color;
	//Color = float4(b - 0.5, 0, 0, 1);
	//Color = sceneDepth / farplane;
	//Color = 1;
	return Color; 
	
}

struct VS_OUT_ILLUM
{
	float4 hPosition	: POSITION;
	float4 cPosition	: TEXCOORD1;
	float2 texCoord		: TEXCOORD0;
	float r				: TEXCOORD2;	
	float4 center		: TEXCOORD3;
	float4 Color		: TEXCOORD4;	
	float2 screenCoord	: TEXCOORD5;
	float4 lightCoord	: TEXCOORD6;
};

VS_OUT_ILLUM HPS_Smoke_L_Depth_Illum_VS (float4 position : POSITION,	
		float4 texCoord : TEXCOORD0,
		float4 Color: COLOR0,
		uniform float width,
		uniform float height,
		uniform float baseRadius,
		uniform float4x4 worldView,
		uniform float4x4 worldViewInv,
		uniform float4x4 lightViewProj,
		uniform float4x4 Proj,
		uniform float4x4 worldViewProj)
{
	VS_OUT_ILLUM OUT;
	//texCoord.z = normalize(texCoord.z);
	//texCoord.w = normalize(texCoord.w);
    float2 offset = texCoord.zw * 100+ baseRadius * 0.000000000001;
	OUT.r = abs(offset.x);
	float4 cPosition;   
	cPosition = mul(worldView, position);
	OUT.center = cPosition;	
	cPosition.xy += offset;	
	OUT.cPosition = cPosition;
	OUT.cPosition.z *= -1;
	//OUT.cPosition.z = OUT.cPosition.z - OUT.r;	
	
    OUT.hPosition = mul( Proj, cPosition );
    float4 sPosition = OUT.hPosition / OUT.hPosition.w;
    OUT.screenCoord = (sPosition.xy + float2(1.0, 1.0)) / float2(2.0, 2.0);
    float2 halfpixel = float2(0.5 / width, 0.5 / height);
    OUT.screenCoord += halfpixel;   
    OUT.screenCoord.y = 1.0 - OUT.screenCoord.y;
         
    OUT.texCoord = texCoord.xy;  
    OUT.Color = Color;
    float4 wPosition = mul(worldViewInv, cPosition);
    OUT.lightCoord = mul(lightViewProj, wPosition);
	return OUT;
} 

float ext(float z, sampler2D illumVolume, float2 lightCoord)
{
	float4 extintion = tex2D(illumVolume, lightCoord);
	
	float intensities[5];
	intensities[0] = 1.0;
	intensities[1] = extintion.r;
	intensities[2] = extintion.g;
	intensities[3] = extintion.b;
	intensities[4] = extintion.a;
	
	z = 4.0 * z;
	int start = floor(z);
	return lerp( intensities[start], intensities[start + 1], z - ((float) start));	
}

float4 HPS_Smoke_L_Depth_Illum_PS(VS_OUT_ILLUM IN,
						uniform float4x4 Proj,
						uniform sampler2D Texture  : register(s0),
						uniform sampler2D depthTexture : register(s1),
						uniform sampler2D illumVolume : register(s2),
						uniform float farplane						
							) : COLOR
{
	float4 Color = 0;
	float4 impostor = tex2D(Texture, IN.texCoord);
	float f = 1.0 - impostor.g;
	float b = impostor.r;
	float alpha = 0;
	
	if(b == 0)
		discard;	

	float sceneDepth = tex2D(depthTexture, IN.screenCoord).r;
	if(sceneDepth == 0)
	{
		alpha = impostor.a;
	}
	else
	{	
		float size = 2.0 * IN.r;
		float frontDepth = IN.cPosition.z + size * (f - 0.5);
		float backDepth = IN.cPosition.z +  size * (b - 0.5);
		
		alpha = (sceneDepth - frontDepth) / (backDepth - frontDepth);
		alpha = saturate(alpha) * impostor.a;
	}
	
	float2 lightCoord;
	lightCoord = (IN.lightCoord.xy + float2(1.0, 1.0)) / 2.0;
	lightCoord.y = 1.0 - lightCoord.y;
	float z = IN.lightCoord.z / IN.lightCoord.w;
	
#ifdef method1
	
		float4 extintion = tex2D(illumVolume, lightCoord);
	
		float intensities[5];
		intensities[0] = 1.0;
		intensities[1] = extintion.r;
		intensities[2] = extintion.g;
		intensities[3] = extintion.b;
		intensities[4] = extintion.a;
			
		float3 start;
		float3 end;
		float3 temp = 1.0;
		float t;
		
		float4 planes = float4(0.33, 0.5, 0.66, 1);
		if(z < planes.x)
		{
			start = intensities[0];
			end = intensities[1];
			t = z / planes.x;
			temp = lerp(start, end, t);				
		}
		if(z > planes.x && z < planes.y)
		{
			start = intensities[1];
			end = intensities[2];
			t = (z - planes.x) / (planes.y - planes.x);
			temp = lerp(start, end, t);					
		}
		if(z > planes.y && z < planes.z)
		{
			start = intensities[2];
			end = intensities[3];
			t = (z - planes.y) / (planes.z - planes.y);
			temp = lerp(start, end, t);				
		}
		if(z > planes.z)
		{
			start = intensities[3];
			end = intensities[4];
			t = (z - planes.z) / (planes.a - planes.z);
			temp = lerp(start, end, t);					
		}
		IN.Color.rgb *= temp;

#else
	
	float4 extintion = tex3D(illumVolume, float3(lightCoord, z));
	IN.Color.rgb *= extintion;
	/*
	float intensities[5];
	intensities[0] = 1.0;
	intensities[1] = extintion.r;
	intensities[2] = extintion.g;
	intensities[3] = extintion.b;
	intensities[4] = extintion.a;
	
	z = 4.0 * z;
	int start = floor(z);
	float ext = lerp( intensities[start], intensities[start + 1], z - ((float) start));	
	IN.Color.rgb *= ext;*/

#endif
	
	Color = float4(1, 1, 1, alpha) * IN.Color;
      return Color; 
	
}


